<?php

namespace Drupal\doorstep_services\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Database\Database;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Render\Markup;
use Drupal\Core\Render\RendererInterface;
use Drupal\Core\Url;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Controller class to display service requests specific to logged-in users.
 */
class DoorstepServicesUserRequestsController extends ControllerBase {

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The renderer.
   *
   * @var \Drupal\Core\Render\RendererInterface
   */
  protected $renderer;

  /**
   * Constructs a DoorstepServicesUserRequestsController object.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   * @param \Drupal\Core\Render\RendererInterface $renderer
   *   The renderer.
   */
  public function __construct(EntityTypeManagerInterface $entity_type_manager, RendererInterface $renderer) {
    $this->entityTypeManager = $entity_type_manager;
    $this->renderer = $renderer;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('entity_type.manager'),
      $container->get('renderer')
    );
  }

  /**
   * Display the service requests specific to the logged-in user.
   */
  public function viewUserRequests() {
    $header = [
      'id' => $this->t('ID'),
      'title' => $this->t('Title'),
      'service_type' => $this->t('Service Type'),
      'status' => $this->t('Status'),
      'contact_number' => $this->t('Contact Number'),
      'email' => $this->t('Email'),
      'notes' => $this->t('Address & Notes'),
      'actions' => $this->t('Actions'),
    ];

    $rows = [];

    // Get the current user ID.
    $current_user_id = $this->currentUser()->id();

    // Fetch nodes with status != 0 and created by the current user.
    $query = $this->entityTypeManager->getStorage('node')->getQuery()
      ->condition('type', 'service_request')
      ->condition('uid', $current_user_id)
      ->sort('created', 'DESC')
      ->accessCheck(FALSE);
    $nids = $query->execute();

    if (empty($nids)) {
      return [
        '#theme' => 'doorstep_services_user_requests',
        '#table' => [
          'header' => [],
          'rows' => [],
        ],
      ];
    }

    $nodes = $this->entityTypeManager->getStorage('node')->loadMultiple($nids);

    // Fetch additional fields data.
    $contact_numbers = Database::getConnection()->select('node__field_contact_number', 'nc')
      ->fields('nc', ['entity_id', 'field_contact_number_value'])
      ->condition('entity_id', $nids, 'IN')
      ->execute()
      ->fetchAllKeyed(0, 1);

    $emails = Database::getConnection()->select('node__field_customer_email', 'ne')
      ->fields('ne', ['entity_id', 'field_customer_email_value'])
      ->condition('entity_id', $nids, 'IN')
      ->execute()
      ->fetchAllKeyed(0, 1);

    $notes = Database::getConnection()->select('node__field_notes', 'nn')
      ->fields('nn', ['entity_id', 'field_notes_value'])
      ->condition('entity_id', $nids, 'IN')
      ->execute()
      ->fetchAllKeyed(0, 1);

    foreach ($nodes as $node) {
      $status_value = $node->get('field_status')->value;
      $status_value = $status_value ?? 'new';

      $status = ucwords(str_replace('_', ' ', $status_value));
      if (empty($status_value)) {
        $status_value = 'New';
      }

      $contact_number = $contact_numbers[$node->id()] ?? $this->t('Not available');
      $email = $emails[$node->id()] ?? $this->t('Not available');
      $note = $notes[$node->id()] ?? $this->t('None');

      // Action links for the service requests.
      $action_links = [];

      if ($status_value === 'cancelled') {
        // Node is inactive, no further actions allowed.
        $action_links['cancelled'] = [
          'data' => [
            '#markup' => $this->t('Request cancelled'),
          ],
        ];
      }

      elseif ($status_value !== 'completed') {
        // Check if the current user has permission to edit this node.
        if ($node->getOwnerId() === $current_user_id) {
          $action_links['edit'] = [
            'data' => [
              '#type' => 'link',
              '#title' => Markup::create($this->t('Edit') . '<br />'),
              '#url' => Url::fromRoute('doorstep_services.edit_service_request', ['node' => $node->id()]),
              '#attributes' => [
                'title' => $this->t('Edit this service request'),
                'class' => ['edit-link'],
              ],
            ],
          ];
        }

        // Provide a cancel link for non-completed requests.
        $action_links['cancel'] = [
          'data' => [
            '#type' => 'link',
            '#title' => $this->t('Cancel Request'),
            '#url' => Url::fromRoute('doorstep_services.cancel_request_form', ['nid' => $node->id()]),
            '#attributes' => [
              'title' => $this->t('Cancel this service request'),
              'class' => ['cancel-link'],
            ],
          ],
        ];
      }
      else {
        $action_links['completed'] = [
          'data' => [
            '#markup' => $this->t('No further actions allowed'),
          ],
        ];
      }

      $rows[] = [
        'id' => $node->id(),
        'title' => $node->label(),
        'service_type' => ucfirst($node->get('field_service_type')->value),
        'status' => $status,
        'contact_number' => $contact_number,
        'email' => $email,
        'notes' => ucfirst($note),
        'actions' => [
          'data' => $action_links,
        ],
      ];
    }
    return [
      '#theme' => 'doorstep_services_user_requests',
      '#table' => [
        'header' => array_values($header),
        'rows' => $rows,
      ],
      '#cache' => ['max-age' => 0],
      '#attached' => [
        'library' => [
          'doorstep_services/doorstep_services_user_requests',
        ],
      ],
    ];

  }

}
