<?php

namespace Drupal\doorstep_services\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Database\Database;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Form\FormBuilderInterface;
use Drupal\Core\Link;
use Drupal\Core\Render\RendererInterface;
use Drupal\Core\Url;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\Request;

/**
 * Controller class to display and manage service requests.
 */
class ServicesController extends ControllerBase {

  /**
   * The form builder service.
   *
   * @var \Drupal\Core\Form\FormBuilderInterface
   */
  protected $formBuilder;

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The renderer.
   *
   * @var \Drupal\Core\Render\RendererInterface
   */
  protected $renderer;

  /**
   * Current request.
   *
   * @var \Symfony\Component\HttpFoundation\Request
   */
  protected $request;

  /**
   * Constructs an ServicesController object.
   *
   * @param \Drupal\Core\Form\FormBuilderInterface $form_builder
   *   The form builder service.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   * @param \Drupal\Core\Render\RendererInterface $renderer
   *   The renderer.
   * @param \Symfony\Component\HttpFoundation\Request $request
   *   Current request.
   */
  public function __construct(FormBuilderInterface $form_builder, EntityTypeManagerInterface $entity_type_manager, RendererInterface $renderer, Request $request) {
    $this->formBuilder = $form_builder;
    $this->entityTypeManager = $entity_type_manager;
    $this->renderer = $renderer;
    $this->request = $request;

  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('form_builder'),
      $container->get('entity_type.manager'),
      $container->get('renderer'),
      $container->get('request_stack')->getCurrentRequest()

    );
  }

  /**
   * Display the service requests.
   */
  public function viewRequests() {
    // Create an instance of the filter form.
    $filter_form = $this->formBuilder->getForm('Drupal\doorstep_services\Form\DoorstepServicesFilterForm');

    // Get the selected status from the form state.
    $selected_status = $this->request->get('status', 'all');

    $header = [
      'id' => $this->t('ID'),
      'title' => $this->t('Title'),
      'service_type' => $this->t('Service Type'),
      'status' => $this->t('Status'),
      'contact_number' => $this->t('Contact Number'),
      'email' => $this->t('Email'),
      'notes' => $this->t('Address & Notes'),
      'actions' => $this->t('Actions'),
    ];

    $rows = [];

    // Fetch nodes based on the selected status filter.
    $query = $this->entityTypeManager->getStorage('node')->getQuery()
      ->condition('type', 'service_request')
      ->sort('created', 'DESC')
      ->accessCheck(FALSE);

    if ($selected_status !== 'all') {
      $query->condition('field_status', $selected_status);
    }

    $nids = $query->execute();
    $nodes = $this->entityTypeManager->getStorage('node')->loadMultiple($nids);

    $contact_numbers = Database::getConnection()->select('node__field_contact_number', 'nc')
      ->fields('nc', ['entity_id', 'field_contact_number_value'])
      ->execute()
      ->fetchAllKeyed(0, 1);

    $emails = Database::getConnection()->select('node__field_customer_email', 'ne')
      ->fields('ne', ['entity_id', 'field_customer_email_value'])
      ->execute()
      ->fetchAllKeyed(0, 1);

    $notes = Database::getConnection()->select('node__field_notes', 'nn')
      ->fields('nn', ['entity_id', 'field_notes_value'])
      ->execute()
      ->fetchAllKeyed(0, 1);

    foreach ($nodes as $node) {
      $status_value = $node->get('field_status')->value;
      $status_value = $status_value ?? 'new';
      $status = ucwords(str_replace('_', ' ', $status_value));

      $contact_number = $contact_numbers[$node->id()] ?? $this->t('Not available');
      $email = $emails[$node->id()] ?? $this->t('Not available');
      $note = $notes[$node->id()] ?? $this->t('None');

      if ($status_value === 'completed') {
        $action_link = $this->t('No further actions allowed');
      }
      elseif ($status_value === 'cancelled') {
        $action_link = $this->t('Request cancelled');
      }
      else {
        $action_link = [
          'data' => [
            '#type' => 'link',
            '#title' => $this->t('Edit'),
            '#url' => Url::fromRoute('doorstep_services.update_status_form', ['nid' => $node->id()]),
            '#attributes' => [
              'title' => $this->t('Edit the status of this service request'),
            ],
          ],
        ];
      }

      $rows[] = [
        'id' => $node->id(),
        'title' => $node->label(),
        'service_type' => ucfirst($node->get('field_service_type')->value),
        'status' => $status,
        'contact_number' => $contact_number,
        'email' => $email,
        'notes' => ucfirst($note),
        'actions' => [
          'data' => $action_link,
        ],
      ];
    }

    // Create the bulk delete link.
    $bulk_delete_url = Url::fromRoute('doorstep_services.bulk_delete_form');
    $bulk_delete_link = Link::fromTextAndUrl($this->t('Delete requests'), $bulk_delete_url)->toRenderable();

    return [
      '#type' => 'container',
    // Add the filter form to the render array.
      'filter_form' => $filter_form,
      'table' => [
        '#type' => 'table',
        '#header' => $header,
        '#rows' => $rows,
      ],
      '#cache' => ['max-age' => 0],
      'bulk_delete' => [
        '#type' => 'markup',
        '#markup' => $this->renderer->renderInIsolation($bulk_delete_link),
      ],
    ];
  }

}
