<?php

namespace Drupal\doorstep_services\Form;

use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Messenger\MessengerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides a form for bulk deleting service requests.
 */
class BulkDeleteForm extends FormBase {

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;
  /**
   * The messenger.
   *
   * @var \Drupal\Core\Messenger\MessengerInterface
   */
  protected $messenger;

  /**
   * Constructs a new BulkDeleteForm.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   * @param \Drupal\Core\Messenger\MessengerInterface $messenger
   *   The messenger.
   */
  public function __construct(EntityTypeManagerInterface $entity_type_manager, MessengerInterface $messenger,) {
    $this->entityTypeManager = $entity_type_manager;
    $this->messenger = $messenger;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('entity_type.manager'),
      $container->get('messenger'),
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'doorstep_services_bulk_delete_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $nodes = $this->entityTypeManager->getStorage('node')->getQuery()
      ->condition('type', 'service_request')
      ->sort('created', 'DESC')
      ->accessCheck(FALSE)
      ->execute();
    $node_storage = $this->entityTypeManager->getStorage('node');

    // Load multiple nodes based.
    $nodes = $node_storage->loadMultiple($nodes);

    $options = [];
    foreach ($nodes as $node) {
      $options[$node->id()] = $node->label();
    }

    $form['select_all'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Select All'),
      '#attributes' => [
        'onclick' => 'jQuery(this).closest("form").find(":checkbox").prop("checked", this.checked);',
      ],
    ];

    $form['nodes'] = [
      '#type' => 'checkboxes',
      '#title' => $this->t('Select requests to delete'),
      '#options' => $options,
    ];

    $form['actions'] = [
      '#type' => 'actions',
    ];

    $form['actions']['submit'] = [
      '#type' => 'submit',
      '#value' => $this->t('Delete Selected'),
      '#button_type' => 'danger',
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $selected_nodes = array_filter($form_state->getValue('nodes'));
    $node_storage = $this->entityTypeManager->getStorage('node');

    if (empty($selected_nodes)) {
      $this->messenger->addWarning($this->t('No requests were selected for deletion.'));
    }
    else {
      foreach ($selected_nodes as $nid) {
        $node = $node_storage->load($nid);
        if ($node) {
          $node->delete();
        }
      }
      $this->messenger->addMessage($this->t('Selected requests have been deleted.'));
    }

    // Redirect back to the view or another relevant page.
    $form_state->setRedirect('doorstep_services.view_requests');
  }

}
