<?php

namespace Drupal\doorstep_services\Form;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Mail\MailManagerInterface;
use Drupal\Core\Routing\RouteMatchInterface;
use Drupal\Core\Session\AccountInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Class DoorstepServiceRequestEditForm.
 *
 * Provides a custom edit form for service requests.
 */
class DoorstepServiceRequestEditForm extends FormBase {

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;
  /**
   * The current route match.
   *
   * @var \Drupal\Core\Routing\RouteMatchInterface
   */
  protected $routeMatch;

  /**
   * The config factory service.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;
  /**
   * The current user.
   *
   * @var \Drupal\Core\Session\AccountInterface
   */
  protected $currentUser;
  /**
   * The mail manager service.
   *
   * @var \Drupal\Core\Mail\MailManagerInterface
   */
  protected $mailManager;

  /**
   * Constructs a new DoorstepServiceRequestEditForm.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   * @param \Drupal\Core\Routing\RouteMatchInterface $route_match
   *   The current route match.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The config factory service.
   * @param \Drupal\Core\Session\AccountInterface $current_user
   *   The current user.
   * @param \Drupal\Core\Mail\MailManagerInterface $mail_manager
   *   The mail manager service.
   */
  public function __construct(EntityTypeManagerInterface $entity_type_manager, RouteMatchInterface $route_match, ConfigFactoryInterface $config_factory, AccountInterface $current_user, MailManagerInterface $mail_manager) {
    $this->entityTypeManager = $entity_type_manager;
    $this->routeMatch = $route_match;
    $this->configFactory = $config_factory;
    $this->currentUser = $current_user;
    $this->mailManager = $mail_manager;

  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('entity_type.manager'),
      $container->get('current_route_match'),
      $container->get('config.factory'),
      $container->get('current_user'),
      $container->get('plugin.manager.mail'),
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'service_request_edit_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state, $node = NULL) {
    // Load the node entity if not already provided.
    if (!$node) {
      $node_storage = $this->entityTypeManager->getStorage('node');
      $node = $node_storage->load($this->routeMatch->getParameter('node'));
    }

    $form['title'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Title'),
      '#default_value' => $node->getTitle(),
      '#required' => TRUE,
    ];

    $form['service_type'] = [
      '#type' => 'select',
      '#title' => $this->t('Service Type'),
      '#default_value' => $node->get('field_service_type')->value,
      '#required' => TRUE,
      '#options' => [
        'doctor' => $this->t('Doctor'),
        'carpenter' => $this->t('Carpenter'),
        'painter' => $this->t('Painter'),
        'fabricator' => $this->t('Fabricator'),
        'ac_technician' => $this->t('AC Technician'),
        'mechanic' => $this->t('Mechanic'),
        'electrician' => $this->t('Electrician'),
        'plumber' => $this->t('Plumber'),
        'fitter' => $this->t('Fitter'),
      ],
    ];
    // Sort the options array alphabetically by the label.
    asort($form['service_type']['#options']);

    $form['contact_number'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Contact Number'),
      '#default_value' => $node->get('field_contact_number')->value,
    ];

    $form['email'] = [
      '#type' => 'email',
      '#title' => $this->t('Email'),
      '#default_value' => $node->get('field_customer_email')->value,
    ];
    $form['preferred_time'] = [
      '#type' => 'date',
      '#title' => $this->t('Preferred Date'),
      '#default_value' => $node->get('field_preferred_time')->value,
      '#required' => TRUE,
      '#min' => date('Y-m-d'),
    ];

    $form['notes'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Address & Notes'),
      '#default_value' => $node->get('field_notes')->value,
    ];

    $form['actions'] = [
      '#type' => 'actions',
    ];
    $form['actions']['submit'] = [
      '#type' => 'submit',
      '#value' => $this->t('Save changes'),
    ];

    $form['#node'] = $node;
    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {

  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    /** @var \Drupal\node\Entity\Node $node */
    $node = $form['#node'];

    $node->setTitle($form_state->getValue('title'));
    $node->set('field_service_type', $form_state->getValue('service_type'));
    $node->set('field_contact_number', $form_state->getValue('contact_number'));
    $node->set('field_customer_email', $form_state->getValue('email'));
    $node->set('field_preferred_time', $form_state->getValue('preferred_time'));
    $node->set('field_notes', $form_state->getValue('notes'));

    $node->save();
    // Prepare the form values for use in the notification.
    $values = $form_state->getValues();

    $this->messenger()->addMessage($this->t('The service request has been updated.'));
    $form_state->setRedirect('doorstep_services.user_requests');

    // Generate the request ID (using node ID as request ID).
    $request_id = $node->id();

    // Send notification email to admin.
    $this->notifyAdmin($values, $request_id);
  }

  /**
   * Notifies the admin about the service requests edit.
   */
  protected function notifyAdmin($values, $request_id) {
    $site_name = $this->configFactory->get('system.site')->get('name');

    $module = 'doorstep_services';
    $key = 'admin_notification';
    $to = $this->configFactory->get('system.site')->get('mail');
    $params['message'] = $this->t('service request (ID: @request_id) has been updated by @name. 
    Service Type: @service_type. 
    Contact: @contact. 
    Email: @email.
    Preferred Date: @preferred_time.

    Best regards,
    @site_name', [
      '@request_id' => $request_id,
      '@name' => $values['email'],
      '@service_type' => $values['service_type'],
      '@contact' => $values['contact_number'],
      '@email' => $values['email'],
      '@preferred_time' => $values['preferred_time'],
      '@site_name' => $site_name,
    ]);
    $params['subject'] = $this->t('Service Request updated (ID: @request_id)', ['@request_id' => $request_id]);
    $langcode = $this->currentUser->getPreferredLangcode();
    $send = TRUE;

    $this->mailManager->mail($module, $key, $to, $langcode, $params, NULL, $send);
  }

}
