<?php

namespace Drupal\doorstep_services\Form;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Mail\MailManagerInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\node\Entity\Node;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\RequestStack;

/**
 * Provides a form to cancel a service request.
 */
class DoorstepServicesCancelForm extends FormBase {

  /**
   * The mail manager service.
   *
   * @var \Drupal\Core\Mail\MailManagerInterface
   */
  protected $mailManager;

  /**
   * The config factory service.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * The current user.
   *
   * @var \Drupal\Core\Session\AccountInterface
   */
  protected $currentUser;

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The request stack service.
   *
   * @var \Symfony\Component\HttpFoundation\RequestStack
   */
  protected $requestStack;

  /**
   * Constructs a new DoorstepServicesCancelForm.
   *
   * @param \Drupal\Core\Mail\MailManagerInterface $mail_manager
   *   The mail manager service.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The config factory service.
   * @param \Drupal\Core\Session\AccountInterface $current_user
   *   The current user.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   * @param \Symfony\Component\HttpFoundation\RequestStack $request_stack
   *   The request stack service.
   */
  public function __construct(MailManagerInterface $mail_manager, ConfigFactoryInterface $config_factory, AccountInterface $current_user, EntityTypeManagerInterface $entity_type_manager, RequestStack $request_stack) {
    $this->mailManager = $mail_manager;
    $this->configFactory = $config_factory;
    $this->currentUser = $current_user;
    $this->entityTypeManager = $entity_type_manager;
    $this->requestStack = $request_stack;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('plugin.manager.mail'),
      $container->get('config.factory'),
      $container->get('current_user'),
      $container->get('entity_type.manager'),
      $container->get('request_stack')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'service_request_cancel_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    // Retrieve the request ID from the route.
    $request = $this->requestStack->getCurrentRequest();
    $request_id = $request->attributes->get('nid');

    $form['request_id'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Service Request ID'),
      '#required' => TRUE,
      '#default_value' => $request_id,
      '#description' => $this->t('ID of the service request you wish to cancel.'),
    ];

    $form['submit'] = [
      '#type' => 'submit',
      '#value' => $this->t('Cancel Request'),
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $request_id = $form_state->getValue('request_id');

    if ($request_id) {
      $node_storage = $this->entityTypeManager->getStorage('node');
      // Load the service request node by request ID.
      $node = $node_storage->load($request_id);

      if ($node) {
        // Set the node status to unpublished (status = 0).
        $node->set('status', 0);

        // Set the field_status field to 'cancelled'.
        $node->set('field_status', 'cancelled');

        // Save the node to apply changes.
        $node->save();

        // Notify the customer and the admin about the cancellation.
        $this->sendCancellationEmail($node);
        $this->notifyAdminCancellation($node);

        // Show a message indicating successful cancellation.
        $this->messenger()->addMessage($this->t('Your service request (ID: @request_id) has been cancelled.', ['@request_id' => $request_id]));
      }
      else {
        $this->messenger()->addError($this->t('The service request could not be found.'));
      }
    }
    $form_state->setRedirect('doorstep_services.user_requests');

  }

  /**
   * Sends a cancellation email to the customer.
   */
  protected function sendCancellationEmail(Node $node) {
    $to = $node->get('field_customer_email')->value;
    $site_name = $this->configFactory->get('system.site')->get('name');
    $request_id = $node->id();

    $module = 'doorstep_services';
    $key = 'customer_cancellation_notification';
    $params['message'] = $this->t('Your service request (ID: @request_id) has been cancelled. 

    Best regards, 
    @site_name', [
      '@request_id' => $request_id,
      '@site_name' => $site_name,
    ]);
    $params['subject'] = $this->t('Service Request Cancellation (ID: @request_id)', ['@request_id' => $request_id]);
    $langcode = $this->currentUser->getPreferredLangcode();
    $send = TRUE;

    $this->mailManager->mail($module, $key, $to, $langcode, $params, NULL, $send);
  }

  /**
   * Sends a notification email to the admin about the cancellation.
   */
  protected function notifyAdminCancellation(Node $node) {
    $to = $this->configFactory->get('system.site')->get('mail');
    $site_name = $this->configFactory->get('system.site')->get('name');
    $request_id = $node->id();

    $module = 'doorstep_services';
    $key = 'admin_cancellation_notification';
    $params['message'] = $this->t('The service request (ID: @request_id) has been cancelled by the customer. 

    Best regards,
    @site_name', [
      '@request_id' => $request_id,
      '@site_name' => $site_name,
    ]);
    $params['subject'] = $this->t('Service Request Cancellation (ID: @request_id)', ['@request_id' => $request_id]);
    $langcode = $this->currentUser->getPreferredLangcode();
    $send = TRUE;

    $this->mailManager->mail($module, $key, $to, $langcode, $params, NULL, $send);
  }

}
