<?php

namespace Drupal\doorstep_services\Form;

use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Url;
use Drupal\doorstep_services\Service\PdfGeneratorService;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\RedirectResponse;

/**
 * Form class to enter the price for a service request.
 */
class PriceEntryForm extends FormBase {

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The PDF generator service.
   *
   * @var \Drupal\doorstep_services\Service\PdfGeneratorService
   */
  protected $pdfGenerator;

  /**
   * Constructs a new PriceEntryForm.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   * @param \Drupal\doorstep_services\Service\PdfGeneratorService $pdf_generator
   *   The PDF generator service.
   */
  public function __construct(EntityTypeManagerInterface $entity_type_manager, PdfGeneratorService $pdf_generator) {
    $this->entityTypeManager = $entity_type_manager;
    $this->pdfGenerator = $pdf_generator;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('entity_type.manager'),
      $container->get('doorstep_services.pdf_generator')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'price_entry_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state, $nid = NULL) {
    // Load the node.
    $node_storage = $this->entityTypeManager->getStorage('node');
    $node = $node_storage->load($nid);
    if (!$node) {
      $this->messenger()->addMessage($this->t('Service ID not found. redirecting!'));
      $url = Url::fromRoute('doorstep_services.view_requests')->toString();
      $response = new RedirectResponse($url);
      $response->send();
      return;
    }
    $form['#attached']['library'][] = 'doorstep_services/doorstep_services';

    $form['nid'] = [
      '#type' => 'hidden',
      '#value' => $nid,
    ];

    // Set the prefix for the price field to the Indian Rupee symbol.
    $form['price'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Service Price'),
      '#required' => TRUE,
      '#default_value' => '',
      '#attributes' => [
        'class' => ['price-input'],
      ],
    ];

    $form['submit'] = [
      '#type' => 'submit',
      '#value' => $this->t('Submit'),
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $nid = $form_state->getValue('nid');
    $price = $form_state->getValue('price');
    $currency = 'INR';

    // Load the node.
    $node_storage = $this->entityTypeManager->getStorage('node');
    $node = $node_storage->load($nid);
    if ($node) {
      // Pass the selected currency type to the PDF generation service.
      $pdf_path = $this->pdfGenerator->generatePdfBill($node, $price, $currency);

      // Send the email with the PDF attachment.
      $this->pdfGenerator->sendBillEmail($node, $pdf_path);

      $this->messenger()->addMessage($this->t('The price has been saved and the bill has been sent.'));
      $form_state->setRedirect('doorstep_services.view_requests');
    }
  }

}
