<?php

namespace Drupal\doorstep_services\Form;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Mail\MailManagerInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\node\Entity\Node;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides a form for service requests.
 */
class RequestForm extends FormBase {

  /**
   * The mail manager service.
   *
   * @var \Drupal\Core\Mail\MailManagerInterface
   */
  protected $mailManager;

  /**
   * The config factory service.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;
  /**
   * The current user.
   *
   * @var \Drupal\Core\Session\AccountInterface
   */
  protected $currentUser;

  /**
   * Constructs a new RequestForm.
   *
   * @param \Drupal\Core\Mail\MailManagerInterface $mail_manager
   *   The mail manager service.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The config factory service.
   * @param \Drupal\Core\Session\AccountInterface $current_user
   *   The current user.
   */
  public function __construct(MailManagerInterface $mail_manager, ConfigFactoryInterface $config_factory, AccountInterface $current_user) {
    $this->mailManager = $mail_manager;
    $this->configFactory = $config_factory;
    $this->currentUser = $current_user;

  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('plugin.manager.mail'),
      $container->get('config.factory'),
      $container->get('current_user')

    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'service_request_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $form['name'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Name'),
      '#required' => TRUE,
    ];

    $form['email'] = [
      '#type' => 'email',
      '#title' => $this->t('Email'),
      '#required' => TRUE,
    ];

    $form['contact_number'] = [
      '#type' => 'tel',
      '#title' => $this->t('Contact Number'),
      '#required' => TRUE,
      '#default_value' => '+91',
      '#attributes' => [
        'pattern' => '\+91[0-9]{10}',
        'placeholder' => '+91XXXXXXXXXX',
      ],
    ];

    $form['service_type'] = [
      '#type' => 'select',
      '#title' => $this->t('Service Type'),
      '#options' => [
        'doctor' => $this->t('Doctor'),
        'carpenter' => $this->t('Carpenter'),
        'painter' => $this->t('Painter'),
        'fabricator' => $this->t('Fabricator'),
        'ac_technician' => $this->t('AC Technician'),
        'mechanic' => $this->t('Mechanic'),
        'electrician' => $this->t('Electrician'),
        'plumber' => $this->t('Plumber'),
        'fitter' => $this->t('Fitter'),
      ],
      '#required' => TRUE,
    ];

    // Sort the options array alphabetically by the label.
    asort($form['service_type']['#options']);

    $form['preferred_time'] = [
      '#type' => 'date',
      '#title' => $this->t('Preferred Date'),
      '#required' => TRUE,
      '#min' => date('Y-m-d'),
    ];

    $form['notes'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Address & Notes'),
      '#description' => $this->t('Add your address & service notes here.'),
    ];

    $form['submit'] = [
      '#type' => 'submit',
      '#value' => $this->t('Submit'),
    ];
    // Attach the custom CSS library.
    $form['#attached']['library'][] = 'doorstep_services/service_request_form';

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $values = $form_state->getValues();

    // Create a new service request node.
    $node = Node::create([
      'type' => 'service_request',
      'title' => $this->t('Service Request from @name', ['@name' => $values['name']]),
      'field_customer_name' => $values['name'],
      'field_customer_email' => $values['email'],
      'field_contact_number' => $values['contact_number'],
      'field_service_type' => $values['service_type'],
      'field_preferred_time' => $values['preferred_time'],
      'field_notes' => $values['notes'],
    ]);
    $node->save();

    // Generate the request ID (using node ID as request ID).
    $request_id = $node->id();

    // Send email to customer.
    $this->sendEmail($values['email'], $values, $request_id);

    // Send notification email to admin.
    $this->notifyAdmin($values, $request_id);

    // Show a confirmation message with email and additional notes.
    $this->messenger()->addMessage($this->t('Your service request (ID: @request_id) has been submitted successfully.', ['@request_id' => $request_id]));
    $form_state->setRedirect('doorstep_services.user_requests');
  }

  /**
   * Sends an email to the customer.
   */
  protected function sendEmail($to, $values, $request_id) {
    $site_name = $this->configFactory->get('system.site')->get('name');

    $module = 'doorstep_services';
    $key = 'customer_notification';
    $params['message'] = $this->t('Thank you @name for your service request (Your request ID is: @request_id). We will contact you shortly. 

    Best regards, 
    @site_name', [
      '@name' => $values['name'],
      '@request_id' => $request_id,
      '@site_name' => $site_name,
    ]);
    $params['subject'] = $this->t('Service Request Confirmation (ID: @request_id)', ['@request_id' => $request_id]);
    $langcode = $this->currentUser->getPreferredLangcode();
    $send = TRUE;

    $this->mailManager->mail($module, $key, $to, $langcode, $params, NULL, $send);
  }

  /**
   * Notifies the admin about the new service request.
   */
  protected function notifyAdmin($values, $request_id) {
    $site_name = $this->configFactory->get('system.site')->get('name');

    $module = 'doorstep_services';
    $key = 'admin_notification';
    $to = $this->configFactory->get('system.site')->get('mail');
    $params['message'] = $this->t('A new service request (ID: @request_id) has been submitted by @name. 
    Service Type: @service_type. 
    Contact: @contact. 
    Email: @email.
    Preferred Date: @preferred_time.

    Best regards,
    @site_name', [
      '@request_id' => $request_id,
      '@name' => $values['name'],
      '@service_type' => $values['service_type'],
      '@contact' => $values['contact_number'],
      '@email' => $values['email'],
      '@preferred_time' => $values['preferred_time'],
      '@site_name' => $site_name,
    ]);
    $params['subject'] = $this->t('New Service Request Submitted (ID: @request_id)', ['@request_id' => $request_id]);
    $langcode = $this->currentUser->getPreferredLangcode();
    $send = TRUE;

    $this->mailManager->mail($module, $key, $to, $langcode, $params, NULL, $send);
  }

}
