<?php

namespace Drupal\doorstep_services\Form;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Database\Connection;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Mail\MailManagerInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\Core\Url;
use Drupal\doorstep_services\Service\PdfGeneratorService;
use Drupal\node\Entity\Node;
use Drupal\node\NodeInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\RedirectResponse;

/**
 * Form class to update the status of a service request.
 */
class StatusUpdateForm extends FormBase {

  /**
   * The mail manager service.
   *
   * @var \Drupal\Core\Mail\MailManagerInterface
   */
  protected $mailManager;

  /**
   * The config factory service.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;
  /**
   * The current user.
   *
   * @var \Drupal\Core\Session\AccountInterface
   */
  protected $currentUser;
  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;
  /**
   * Database connection.
   */
  protected Connection $database;

  /**
   * The PDF generator service.
   *
   * @var \Drupal\doorstep_services\Service\PdfGeneratorService
   */
  protected $pdfGenerator;

  /**
   * Constructs a new StatusUpdateForm.
   *
   * @param \Drupal\Core\Mail\MailManagerInterface $mail_manager
   *   The mail manager service.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The config factory service.
   * @param \Drupal\Core\Session\AccountInterface $current_user
   *   The current user.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   * @param \Drupal\Core\Database\Connection $database
   *   Database connection.
   * @param \Drupal\doorstep_services\Service\PdfGeneratorService $pdf_generator
   *   The PDF generator service.
   */
  public function __construct(MailManagerInterface $mail_manager, ConfigFactoryInterface $config_factory, AccountInterface $current_user, EntityTypeManagerInterface $entity_type_manager, Connection $database, PdfGeneratorService $pdf_generator) {
    $this->mailManager = $mail_manager;
    $this->configFactory = $config_factory;
    $this->currentUser = $current_user;
    $this->entityTypeManager = $entity_type_manager;
    $this->database = $database;
    $this->pdfGenerator = $pdf_generator;

  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('plugin.manager.mail'),
      $container->get('config.factory'),
      $container->get('current_user'),
      $container->get('entity_type.manager'),
      $container->get('database'),
      $container->get('doorstep_services.pdf_generator')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'status_update_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state, $nid = NULL) {
    $node_storage = $this->entityTypeManager->getStorage('node');
    $node = $node_storage->load($nid);

    if (!$node) {
      $this->messenger()->addMessage($this->t('Service ID not found. redirecting!'));
      $url = Url::fromRoute('doorstep_services.view_requests')->toString();
      $response = new RedirectResponse($url);
      $response->send();
      return;
    }

    $status_value = $node->get('field_status')->value;
    $status_options = [
      'pending' => $this->t('Pending'),
      'in_progress' => $this->t('In Progress'),
      'completed' => $this->t('Completed'),
      'cancelled' => $this->t('Cancel'),
    ];

    $form['nid'] = [
      '#type' => 'hidden',
      '#value' => $nid,
    ];

    $form['status'] = [
      '#type' => 'select',
      '#title' => $this->t('Status'),
      '#options' => $status_options,
      '#default_value' => $status_value,
    ];

    // Add the "Assign To" field for admins and make it conditional.
    if ($this->currentUser()->hasPermission('administer site configuration')) {
      $field_assigned_to_vocabulary_ids = $node->get('field_assigned_to')->getFieldDefinition()->getSetting('target_type');

      $form['assign_to'] = [
        '#type' => 'entity_autocomplete',
        '#title' => $this->t('Assign To'),
        '#target_type' => 'taxonomy_term',
        '#selection_settings' => [
          'target_bundles' => $field_assigned_to_vocabulary_ids,
        ],
        '#default_value' => $this->getAssignedToDefaultValue($node),
        '#required' => FALSE,
        '#description' => $this->t('Assign this service request to a specific service provider.'),
        '#states' => [
          'visible' => [
            ':input[name="status"]' => ['value' => 'in_progress'],
          ],
        ],
      ];
    }

    $form['submit'] = [
      '#type' => 'submit',
      '#value' => $this->t('Save'),
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $nid = $form_state->getValue('nid');
    $status = $form_state->getValue('status');
    $assign_to = $form_state->getValue('assign_to');

    // Update the node's status field in the database.
    $this->database->update('node__field_status')
      ->fields(['field_status_value' => $status])
      ->condition('entity_id', $nid)
      ->execute();

    // Load and save the node.
    $node_storage = $this->entityTypeManager->getStorage('node');
    $node = $node_storage->load($nid);
    if ($node) {
      $node->set('field_status', $status);
      // Assign to a designated person if set.
      if ($this->currentUser->hasPermission('administer site configuration') && !empty($assign_to)) {
        $node->set('field_assigned_to', $assign_to);
      }
      $node->save();

      // Trigger email notification on status change.
      $this->sendStatusUpdateEmail($node);

      // Redirect to price entry form if the status is completed.
      if ($status === 'completed') {
        $form_state->setRedirect('doorstep_services.price_entry_form', ['nid' => $nid]);
        return;
      }
    }

    // Handle cancellation case.
    if ($status === 'cancelled') {
      $this->messenger()->addMessage($this->t('The service request has been cancelled.'));
    }
    else {
      $this->messenger()->addMessage($this->t('The status has been updated.'));
    }

    // Redirect to the request view page after updating status.
    $form_state->setRedirect('doorstep_services.view_requests');
  }

  /**
   * Sends an email with the PDF bill to the customer.
   *
   * @param \Drupal\node\Entity\Node $node
   *   The service request node.
   * @param string $pdf_path
   *   The file path of the PDF to attach.
   */
  protected function sendBillEmail(Node $node, $pdf_path) {
    $mailManager = $this->mailManager;
    $module = 'doorstep_services';
    $key = 'bill_generated';
    $to = $node->get('field_customer_email')->value;

    // Retrieve the site name from the configuration.
    $site_name = $this->configFactory->get('system.site')->get('name');

    $params['subject'] = $this->t('Your Service Request Bill');
    $params['message'] = $this->t('Hello @name,
  
        Please find attached the bill for your completed service request.
  
        Thank you for choosing @site_name.
  
        Best regards,
        @site_name Team', [
          '@name' => $node->get('field_customer_email')->value,
          '@site_name' => $site_name,
        ]);

    // Attach the PDF.
    $params['attachments'][] = [
      'filecontent' => file_get_contents($pdf_path),
      'filename' => basename($pdf_path),
      'filemime' => 'application/pdf',
    ];

    $langcode = $this->currentUser->getPreferredLangcode();
    $send = TRUE;

    $result = $mailManager->mail($module, $key, $to, $langcode, $params, NULL, $send);

    if ($result['result'] !== TRUE) {
      $this->messenger()->addError($this->t('There was a problem sending the bill email to the customer.'));
    }
  }

  /**
   * Sends an email to the customer notifying them of the status update.
   *
   * @param \Drupal\node\Entity\Node $node
   *   The service request node.
   */
  protected function sendStatusUpdateEmail(Node $node) {
    $mailManager = $this->mailManager;
    $module = 'doorstep_services';
    $key = 'status_update';
    $to = $node->get('field_customer_email')->value;

    // Retrieve the site name from the configuration.
    $site_name = $this->configFactory->get('system.site')->get('name');
    // Fetch the assigned provider's name.
    $assigned_provider = '';
    if ($assigned_to_tid = $node->get('field_assigned_to')->target_id) {
      $term = $this->entityTypeManager->getStorage('taxonomy_term')->load($assigned_to_tid);
      if ($term) {
        $assigned_provider = $term->getName();
      }
    }
    $status = ucwords(str_replace('_', ' ', $node->get('field_status')->value));

    $params['subject'] = $this->t('Your Service Request Status Update');
    $params['message'] = $this->t('Hello @name,

      Your service request for "@service_type" has been updated to: @status.
      It has been assigned to: @assigned_provider.
      Thank you for choosing @site_name.

      Best regards,
        @site_name Team', [
          '@name' => $node->get('field_customer_email')->value,
          '@service_type' => $node->get('field_service_type')->value,
          '@status' => $status,
          '@site_name' => $site_name,
          '@assigned_provider' => $assigned_provider,
        ]);

    $langcode = $this->currentUser->getPreferredLangcode();
    $send = TRUE;

    $result = $mailManager->mail($module, $key, $to, $langcode, $params, NULL, $send);

    if ($result['result'] !== TRUE) {
      $this->messenger()->addError($this->t('There was a problem sending the notification email to the customer.'));
    }
  }

  /**
   * Gets the default value for the "Assign To" field.
   *
   * @param \Drupal\node\NodeInterface $node
   *   The node entity for which the form is being built.
   *
   * @return \Drupal\taxonomy\Entity\Term|null
   *   The term entity if it exists, otherwise NULL.
   */
  protected function getAssignedToDefaultValue(NodeInterface $node) {
    if ($node->hasField('field_assigned_to') && !$node->get('field_assigned_to')->isEmpty()) {
      // Load the term entity using the term ID from the field_assigned_to field.
      $term_id = $node->get('field_assigned_to')->target_id;
      return $this->entityTypeManager->getStorage('taxonomy_term')->load($term_id);
    }
    return NULL;
  }

}
