<?php

namespace Drupal\doorstep_services\Service;

use chillerlan\QRCode\QRCode;
use chillerlan\QRCode\QROptions;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Extension\ModuleExtensionList;
use Drupal\Core\File\FileSystemInterface;
use Drupal\Core\Mail\MailManagerInterface;
use Drupal\Core\Messenger\MessengerInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\Core\StringTranslation\TranslationInterface;
use Drupal\node\Entity\Node;

/**
 * Provides PDF generation functionality.
 */
class PdfGeneratorService {

  /**
   * The mail manager service.
   *
   * @var \Drupal\Core\Mail\MailManagerInterface
   */
  protected $mailManager;

  /**
   * The config factory service.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;
  /**
   * The current user.
   *
   * @var \Drupal\Core\Session\AccountInterface
   */
  protected $currentUser;

  /**
   * The messenger.
   *
   * @var \Drupal\Core\Messenger\MessengerInterface
   */
  protected $messenger;

  /**
   * The string translation service.
   *
   * @var \Drupal\Core\StringTranslation\TranslationInterface
   */
  protected $translation;
  /**
   * The module extension list.
   *
   * @var \Drupal\Core\Extension\ModuleExtensionList
   */
  protected $moduleExtensionList;
  /**
   * The file system service.
   *
   * @var \Drupal\Core\File\FileSystemInterface
   */
  protected $fileSystem;

  /**
   * Constructs a new PdfGeneratorService.
   *
   * @param \Drupal\Core\Mail\MailManagerInterface $mail_manager
   *   The mail manager service.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The config factory service.
   * @param \Drupal\Core\Session\AccountInterface $current_user
   *   The current user.
   * @param \Drupal\Core\Messenger\MessengerInterface $messenger
   *   The messenger.
   * @param \Drupal\Core\StringTranslation\TranslationInterface $translation
   *   The string translation service.
   * @param \Drupal\Core\Extension\ModuleExtensionList $extension_list_module
   *   The module extension list.
   * @param \Drupal\Core\File\FileSystemInterface $file_system
   *   The file system service.
   */
  public function __construct(MailManagerInterface $mail_manager, ConfigFactoryInterface $config_factory, AccountInterface $current_user, MessengerInterface $messenger, TranslationInterface $translation, ModuleExtensionList $extension_list_module, FileSystemInterface $file_system) {
    $this->mailManager = $mail_manager;
    $this->configFactory = $config_factory;
    $this->currentUser = $current_user;
    $this->messenger = $messenger;
    $this->translation = $translation;
    $this->moduleExtensionList = $extension_list_module;
    $this->fileSystem = $file_system;
  }

  /**
   * Generate a PDF bill for a service request.
   *
   * @param \Drupal\node\Entity\Node $node
   *   The service request node.
   * @param string $price
   *   The price of the service.
   * @param string $currency
   *   The currency type of the service.
   *
   * @return string
   *   The file path of the generated PDF.
   */
  public function generatePdfBill(Node $node, $price, $currency = 'INR') {
    // Check if price is provided.
    if (empty($price)) {
      throw new \InvalidArgumentException('Price must be provided.');
    }

    // Initialize TCPDF object.
    $pdf = new \TCPDF();

    // Set document information.
    $pdf->SetCreator(PDF_CREATOR);
    $pdf->SetAuthor($this->configFactory->get('system.site')->get('name'));
    $pdf->SetTitle('Service Request Bill');
    $pdf->SetSubject('Bill for Completed Service Request');
    $pdf->SetKeywords('Service, Bill, PDF');

    // Get site name, slogan, and email from configuration.
    $site_name = $this->configFactory->get('system.site')->get('name');
    $site_slogan = $this->configFactory->get('system.site')->get('slogan');
    $site_email = $this->configFactory->get('system.site')->get('mail');

    // Set default header data using site name, slogan, and email.
    $pdf->SetHeaderData('', 0, $site_name, $site_slogan . "\n" . $site_email);

    // Set header and footer fonts.
    $pdf->setHeaderFont(['dejavusans', '', 10]);
    $pdf->setFooterFont(['dejavusans', '', 8]);

    $pdf->SetMargins(15, 27, 15);
    $pdf->SetHeaderMargin(5);
    $pdf->SetFooterMargin(10);
    $pdf->SetAutoPageBreak(TRUE, 25);
    $pdf->SetFont('dejavusans', '', 12);
    $pdf->AddPage();

    // Load CSS.
    $module_path = $this->moduleExtensionList->getPath('doorstep_services');
    $css_file_path = $module_path . '/css/pdf_styles.css';
    $css = file_exists($css_file_path) ? file_get_contents($css_file_path) : '';

    $html = '<style>' . $css . '</style>
    <div class="bill-container">
      <div class="bill-header">Service Request Bill</div>
      <div class="bill-item">
        <span class="bill-label">Request ID:</span>
        <span class="bill-value">' . $node->id() . '</span>
      </div>
      <div class="bill-item">
        <span class="bill-label">Service Type:</span>
        <span class="bill-value">' . ucfirst($node->get('field_service_type')->value) . '</span>
      </div>
      <div class="bill-item">
        <span class="bill-label">Status:</span>
        <span class="bill-value">' . ucfirst($node->get('field_status')->value) . '</span>
      </div>
      <div class="bill-item">
        <span class="bill-label">Contact Number:</span>
        <span class="bill-value">' . $node->get('field_contact_number')->value . '</span>
      </div>
      <div class="bill-item">
        <span class="bill-label">Email:</span>
        <span class="bill-value">' . $node->get('field_customer_email')->value . '</span>
      </div>
      <div class="bill-item">
        <span class="bill-label">Address & Notes:</span>
        <span class="bill-value">' . $node->get('field_notes')->value . '</span>
      </div>
      <div class="bill-item">
        <span class="bill-label">Service Charges:</span>
        <span class="bill-value">' . $currency . ' ' . $price . '</span>
      </div>
    </div>';

    $pdf->writeHTML($html, TRUE, FALSE, TRUE, FALSE, '');
    $pdf_path = 'public://bill_' . $node->id() . '.pdf';
    $pdf->Output($pdf_path, 'F');

    return $pdf_path;
  }

  /**
   * Send an email with the generated PDF bill.
   *
   * @param \Drupal\node\Entity\Node $node
   *   The service request node.
   * @param string $pdf_path
   *   The file path of the generated PDF.
   */
  public function sendBillEmail(Node $node, $pdf_path) {
    $mailManager = $this->mailManager;
    $module = 'doorstep_services';
    $key = 'bill_generated';

    $customer_email = $node->get('field_customer_email')->value;
    $admin_email = $this->configFactory->get('system.site')->get('mail');
    $site_name = $this->configFactory->get('system.site')->get('name');
    $upi_id = $this->configFactory->get('doorstep_services.settings')->get('upi_id');

    // Generate UPI URI for the QR code.
    $upi_uri = "upi://pay?pa={$upi_id}&pn=Doorstep%20Services&cu=INR";

    // Configure QR code options.
    $options = new QROptions([
      'eccLevel' => QRCode::ECC_H,
      'outputType' => QRCode::OUTPUT_IMAGE_PNG,
      'imageBase64' => FALSE,
      'scale' => 5,
    ]);

    // Create QR code and save to temporary file.
    $qrCode = new QRCode($options);
    $qr_image_data = $qrCode->render($upi_uri);

    // Save QR code to a temporary file.
    $qr_code_path = 'temporary://upi-qr-code.png';
    file_put_contents($qr_code_path, $qr_image_data);

    $params = [
      'subject' => $this->translation->translate('Your Service Request Bill'),
      'message' => $this->translation->translate('Hello @name,
    
      Please find attached the bill for your completed service request.
      You can make an online payment using the attached QR code.
  
      Thank you for choosing @site_name.
  
      Best regards,
      @site_name Team', [
        '@name' => $customer_email,
        '@site_name' => $site_name,
      ]),
      'attachments' => [
        [
          'filecontent' => file_get_contents($pdf_path),
          'filename' => basename($pdf_path),
          'filemime' => 'application/pdf',
        ],
        [
          'filecontent' => file_get_contents($qr_code_path),
          'filename' => 'upi-qr-code.png',
          'filemime' => 'image/png',
        ],
      ],
    ];

    $langcode = $this->currentUser->getPreferredLangcode();
    $send = TRUE;

    // Send email to customer.
    $result = $mailManager->mail($module, $key, $customer_email, $langcode, $params, NULL, $send);

    if ($result['result'] !== TRUE) {
      $this->messenger->addError('There was a problem sending the bill email to the customer.');
    }

    $request_id = $node->id();

    // Send a copy of the bill to the site admin.
    $admin_params = $params;
    $admin_params['message'] = $this->translation->translate('Dear Admin,

      Please find attached a copy of the bill for the completed service request with request ID @request_id by @name.
      Best regards,
      @site_name Team', [
        '@request_id' => $request_id,
        '@name' => $customer_email,
        '@site_name' => $site_name,
      ]);

    $admin_result = $mailManager->mail($module, $key, $admin_email, $langcode, $admin_params, NULL, $send);

    if ($admin_result['result'] !== TRUE) {
      $this->messenger->addError('There was a problem sending the bill email to the site admin.');
    }

    // Clean up the temporary file.
    $this->fileSystem->delete($qr_code_path);
  }

}
